@@ -7,6 +7,8 @@ import time
 import urlparse
 import urllib
 import httplib2
+import logging
+
 try:
     import json
 except ImportError:
@@ -17,19 +19,24 @@ if not hasattr(urlparse, 'parse_qsl'):
     import cgi
     urlparse.parse_qsl = cgi.parse_qsl
 
-import novaclient
+
 from novaclient import exceptions
 
 
-class OpenStackClient(httplib2.Http):
+_logger = logging.getLogger(__name__)
 
-    USER_AGENT = 'python-novaclient/%s' % novaclient.__version__
 
-    def __init__(self, user, apikey, auth_url):
-        super(OpenStackClient, self).__init__()
+class HTTPClient(httplib2.Http):
+
+    USER_AGENT = 'python-novaclient'
+
+    def __init__(self, user, apikey, projectid, auth_url, timeout=None):
+        super(HTTPClient, self).__init__(timeout=timeout)
         self.user = user
         self.apikey = apikey
+        self.projectid = projectid
         self.auth_url = auth_url
+        self.version = 'v1.0'
 
         self.management_url = None
         self.auth_token = None
@@ -37,6 +44,24 @@ class OpenStackClient(httplib2.Http):
         # httplib2 overrides
         self.force_exception_to_status_code = True
 
+    def http_log(self, args, kwargs, resp, body):
+        if not _logger.isEnabledFor(logging.DEBUG):
+            return
+
+        string_parts = ['curl -i']
+        for element in args:
+            if element in ('GET', 'POST'):
+                string_parts.append(' -X %s' % element)
+            else:
+                string_parts.append(' %s' % element)
+
+        for element in kwargs['headers']:
+            header = ' -H "%s: %s"' % (element, kwargs['headers'][element])
+            string_parts.append(header)
+
+        _logger.debug("REQ: %s\n" % "".join(string_parts))
+        _logger.debug("RESP:%s %s\n", resp, body)
+
     def request(self, *args, **kwargs):
         kwargs.setdefault('headers', {})
         kwargs['headers']['User-Agent'] = self.USER_AGENT
@@ -44,12 +69,10 @@ class OpenStackClient(httplib2.Http):
             kwargs['headers']['Content-Type'] = 'application/json'
             kwargs['body'] = json.dumps(kwargs['body'])
 
-        if httplib2.debuglevel == 1:
-            print "ARGS:", args
-        resp, body = super(OpenStackClient, self).request(*args, **kwargs)
-        if httplib2.debuglevel == 1:
-            print "RESPONSE", resp
-            print "BODY", body
+        resp, body = super(HTTPClient, self).request(*args, **kwargs)
+
+        self.http_log(args, kwargs, resp, body)
+
         if body:
             try:
                 body = json.loads(body)
@@ -58,7 +81,7 @@ class OpenStackClient(httplib2.Http):
         else:
             body = None
 
-        if resp.status in (400, 401, 403, 404, 413, 500, 501):
+        if resp.status in (400, 401, 403, 404, 408, 413, 500, 501):
             raise exceptions.from_response(resp, body)
 
         return resp, body
@@ -72,6 +95,9 @@ class OpenStackClient(httplib2.Http):
         # re-authenticate and try again. If it still fails, bail.
         try:
             kwargs.setdefault('headers', {})['X-Auth-Token'] = self.auth_token
+            if self.projectid:
+                kwargs['headers']['X-Auth-Project-Id'] = self.projectid
+
             resp, body = self.request(self.management_url + url, method,
                                       **kwargs)
             return resp, body
@@ -98,16 +124,28 @@ class OpenStackClient(httplib2.Http):
         return self._cs_request(url, 'DELETE', **kwargs)
 
     def authenticate(self):
-        headers = {'X-Auth-User': self.user, 'X-Auth-Key': self.apikey}
+        scheme, netloc, path, query, frag = urlparse.urlsplit(
+                                                    self.auth_url)
+        path_parts = path.split('/')
+        for part in path_parts:
+            if len(part) > 0 and part[0] == 'v':
+                self.version = part
+                break
+
+        headers = {'X-Auth-User': self.user,
+                   'X-Auth-Key': self.apikey}
+        if self.projectid:
+            headers['X-Auth-Project-Id'] = self.projectid
         resp, body = self.request(self.auth_url, 'GET', headers=headers)
         self.management_url = resp['x-server-management-url']
+
         self.auth_token = resp['x-auth-token']
 
     def _munge_get_url(self, url):
         """
         Munge GET URLs to always return uncached content.
 
-        The OpenStack Nova API caches data *very* agressively and doesn't
+        The OpenStack Compute API caches data *very* agressively and doesn't
         respect cache headers. To avoid stale data, then, we append a little
         bit of nonsense onto GET parameters; this appears to force the data not
         to be cached.
