@@ -21,16 +21,69 @@ import uuid
 
 from novaclient import exceptions
 from novaclient import utils
-from novaclient.v1_1 import client
-from novaclient.v1_1 import servers
+from novaclient.v1_0 import client
+from novaclient.v1_0 import backup_schedules
+from novaclient.v1_0 import servers
 
 
 CLIENT_CLASS = client.Client
 
+# Choices for flags.
+DAY_CHOICES = [getattr(backup_schedules, i).lower()
+               for i in dir(backup_schedules)
+               if i.startswith('BACKUP_WEEKLY_')]
+HOUR_CHOICES = [getattr(backup_schedules, i).lower()
+                for i in dir(backup_schedules)
+                if i.startswith('BACKUP_DAILY_')]
 
+
+# Sentinal for boot --key
 AUTO_KEY = object()
 
 
+@utils.arg('server', metavar='<server>', help='Name or ID of server.')
+@utils.arg('--enable', dest='enabled', default=None, action='store_true',
+                                               help='Enable backups.')
+@utils.arg('--disable', dest='enabled', action='store_false',
+                                  help='Disable backups.')
+@utils.arg('--weekly', metavar='<day>', choices=DAY_CHOICES,
+     help='Schedule a weekly backup for <day> (one of: %s).' %
+                              utils.pretty_choice_list(DAY_CHOICES))
+@utils.arg('--daily', metavar='<time-window>', choices=HOUR_CHOICES,
+     help='Schedule a daily backup during <time-window> (one of: %s).' %
+                                       utils.pretty_choice_list(HOUR_CHOICES))
+def do_backup_schedule(cs, args):
+    """
+    Show or edit the backup schedule for a server.
+
+    With no flags, the backup schedule will be shown. If flags are given,
+    the backup schedule will be modified accordingly.
+    """
+    server = _find_server(cs, args.server)
+
+    # If we have some flags, update the backup
+    backup = {}
+    if args.daily:
+        backup['daily'] = getattr(backup_schedules, 'BACKUP_DAILY_%s' %
+                                                    args.daily.upper())
+    if args.weekly:
+        backup['weekly'] = getattr(backup_schedules, 'BACKUP_WEEKLY_%s' %
+                                                     args.weekly.upper())
+    if args.enabled is not None:
+        backup['enabled'] = args.enabled
+    if backup:
+        server.backup_schedule.update(**backup)
+    else:
+        utils.print_dict(server.backup_schedule._info)
+
+@utils.arg('server', metavar='<server>', help='Name or ID of server.')
+def do_backup_schedule_delete(cs, args):
+    """
+    Delete the backup schedule for a server.
+    """
+    server = _find_server(cs, args.server)
+    server.backup_schedule.delete()
+
 def _boot(cs, args, reservation_id=None, min_count=None, max_count=None):
     """Boot a new server."""
     if min_count is None:
@@ -46,6 +99,13 @@ def _boot(cs, args, reservation_id=None, min_count=None, max_count=None):
     image = args.image or cs.images.find(name="Ubuntu 10.04 LTS "\
                                                    "(lucid)")
 
+    # Map --ipgroup <name> to an ID.
+    # XXX do this for flavor/image?
+    if args.ipgroup:
+        ipgroup = _find_ipgroup(cs, args.ipgroup)
+    else:
+        ipgroup = None
+
     metadata = dict(v.split('=') for v in args.meta)
 
     files = {}
@@ -77,7 +137,7 @@ def _boot(cs, args, reservation_id=None, min_count=None, max_count=None):
         except IOError, e:
             raise exceptions.CommandError("Can't open '%s': %s" % (keyfile, e))
 
-    return (args.name, image, flavor, metadata, files,
+    return (args.name, image, flavor, ipgroup, metadata, files,
             reservation_id, min_count, max_count)
 
 @utils.arg('--flavor',
@@ -90,6 +150,10 @@ def _boot(cs, args, reservation_id=None, min_count=None, max_count=None):
      metavar='<image>',
      help="Image ID (see 'nova images'). "\
           "Defaults to Ubuntu 10.04 LTS.")
+@utils.arg('--ipgroup',
+     default=None,
+     metavar='<group>',
+     help="IP group name or ID (see 'nova ipgroup-list').")
 @utils.arg('--meta',
      metavar="<key=value>",
      action='append',
@@ -113,17 +177,17 @@ def _boot(cs, args, reservation_id=None, min_count=None, max_count=None):
 @utils.arg('name', metavar='<name>', help='Name for the new server')
 def do_boot(cs, args):
     """Boot a new server."""
-    name, image, flavor, metadata, files, reservation_id, \
+    name, image, flavor, ipgroup, metadata, files, reservation_id, \
                 min_count, max_count = _boot(cs, args)
 
     server = cs.servers.create(args.name, image, flavor,
+                                    ipgroup=ipgroup,
                                     meta=metadata,
                                     files=files,
                                     min_count=min_count,
                                     max_count=max_count)
     utils.print_dict(server._info)
 
-
 @utils.arg('--flavor',
      default=None,
      metavar='<flavor>',
@@ -134,6 +198,59 @@ def do_boot(cs, args):
      metavar='<image>',
      help="Image ID (see 'nova images'). "\
           "Defaults to Ubuntu 10.04 LTS.")
+@utils.arg('--ipgroup',
+     default=None,
+     metavar='<group>',
+     help="IP group name or ID (see 'nova ipgroup-list').")
+@utils.arg('--meta',
+     metavar="<key=value>",
+     action='append',
+     default=[],
+     help="Record arbitrary key/value metadata. "\
+          "May be give multiple times.")
+@utils.arg('--file',
+     metavar="<dst-path=src-path>",
+     action='append',
+     dest='files',
+     default=[],
+     help="Store arbitrary files from <src-path> locally to <dst-path> "\
+          "on the new server. You may store up to 5 files.")
+@utils.arg('--key',
+     metavar='<path>',
+     nargs='?',
+     const=AUTO_KEY,
+     help="Key the server with an SSH keypair. "\
+          "Looks in ~/.ssh for a key, "\
+          "or takes an explicit <path> to one.")
+@utils.arg('account', metavar='<account>', help='Account to build this'\
+     ' server for')
+@utils.arg('name', metavar='<name>', help='Name for the new server')
+def do_boot_for_account(cs, args):
+    """Boot a new server in an account."""
+    name, image, flavor, ipgroup, metadata, files, reservation_id, \
+            min_count, max_count = _boot(cs, args)
+
+    server = cs.accounts.create_instance_for(args.account, args.name,
+                image, flavor,
+                ipgroup=ipgroup,
+                meta=metadata,
+                files=files)
+    utils.print_dict(server._info)
+
+@utils.arg('--flavor',
+     default=None,
+     metavar='<flavor>',
+     help="Flavor ID (see 'nova flavors'). "\
+          "Defaults to 256MB RAM instance.")
+@utils.arg('--image',
+     default=None,
+     metavar='<image>',
+     help="Image ID (see 'nova images'). "\
+          "Defaults to Ubuntu 10.04 LTS.")
+@utils.arg('--ipgroup',
+     default=None,
+     metavar='<group>',
+     help="IP group name or ID (see 'nova ipgroup-list').")
 @utils.arg('--meta',
      metavar="<key=value>",
      action='append',
@@ -177,7 +294,7 @@ def do_zone_boot(cs, args):
     reservation_id = args.reservation_id
     min_count = args.min_instances
     max_count = args.max_instances
-    name, image, flavor, metadata, \
+    name, image, flavor, ipgroup, metadata, \
             files, reservation_id, min_count, max_count = \
                              _boot(cs, args,
                                         reservation_id=reservation_id,
@@ -185,6 +302,7 @@ def do_zone_boot(cs, args):
                                         max_count=max_count)
 
     reservation_id = cs.zones.boot(args.name, image, flavor,
+                                        ipgroup=ipgroup,
                                         meta=metadata,
                                         files=files,
                                         reservation_id=reservation_id,
@@ -192,7 +310,6 @@ def do_zone_boot(cs, args):
                                         max_count=max_count)
     print "Reservation ID=", reservation_id
 
-
 def _translate_flavor_keys(collection):
     convert = [('ram', 'memory_mb'), ('disk', 'local_gb')]
     for item in collection:
@@ -201,7 +318,6 @@ def _translate_flavor_keys(collection):
             if from_key in keys and to_key not in keys:
                 setattr(item, to_key, item._info[from_key])
 
-
 def do_flavor_list(cs, args):
     """Print a list of available 'flavors' (sizes of servers)."""
     flavors = cs.flavors.list()
@@ -220,6 +336,14 @@ def do_image_list(cs, args):
     """Print a list of available images to boot from."""
     utils.print_list(cs.images.list(), ['ID', 'Name', 'Status'])
 
+@utils.arg('server', metavar='<server>', help='Name or ID of server.')
+@utils.arg('name', metavar='<name>', help='Name of snapshot.')
+def do_image_create(cs, args):
+    """Create a new image by taking a snapshot of a running server."""
+    server = _find_server(cs, args.server)
+    image = cs.images.create(server, args.name)
+    utils.print_dict(image._info)
+
 @utils.arg('image', metavar='<image>', help='Name or ID of image.')
 def do_image_delete(cs, args):
     """
@@ -231,6 +355,56 @@ def do_image_delete(cs, args):
     image = _find_image(cs, args.image)
     image.delete()
 
+@utils.arg('server', metavar='<server>', help='Name or ID of server.')
+@utils.arg('group', metavar='<group>', help='Name or ID of group.')
+@utils.arg('address', metavar='<address>', help='IP address to share.')
+def do_ip_share(cs, args):
+    """Share an IP address from the given IP group onto a server."""
+    server = _find_server(cs, args.server)
+    group = _find_ipgroup(cs, args.group)
+    server.share_ip(group, args.address)
+
+@utils.arg('server', metavar='<server>', help='Name or ID of server.')
+@utils.arg('address', metavar='<address>',
+                help='Shared IP address to remove from the server.')
+def do_ip_unshare(cs, args):
+    """Stop sharing an given address with a server."""
+    server = _find_server(cs, args.server)
+    server.unshare_ip(args.address)
+
+def do_ipgroup_list(cs, args):
+    """Show IP groups."""
+    def pretty_server_list(ipgroup):
+        return ", ".join(cs.servers.get(id).name
+                         for id in ipgroup.servers)
+
+    utils.print_list(cs.ipgroups.list(),
+               fields=['ID', 'Name', 'Server List'],
+               formatters={'Server List': pretty_server_list})
+
+@utils.arg('group', metavar='<group>', help='Name or ID of group.')
+def do_ipgroup_show(cs, args):
+    """Show details about a particular IP group."""
+    group = _find_ipgroup(cs, args.group)
+    utils.print_dict(group._info)
+
+@utils.arg('name', metavar='<name>', help='What to name this new group.')
+@utils.arg('server', metavar='<server>', nargs='?',
+     help='Server (name or ID) to make a member of this new group.')
+def do_ipgroup_create(cs, args):
+    """Create a new IP group."""
+    if args.server:
+        server = _find_server(cs, args.server)
+    else:
+        server = None
+    group = cs.ipgroups.create(args.name, server)
+    utils.print_dict(group._info)
+
+@utils.arg('group', metavar='<group>', help='Name or ID of group.')
+def do_ipgroup_delete(cs, args):
+    """Delete an IP group."""
+    _find_ipgroup(cs, args.group).delete()
+
 @utils.arg('--fixed_ip',
     dest='fixed_ip',
     metavar='<fixed_ip>',
@@ -286,28 +460,12 @@ def do_list(cs, args):
             'name': args.name,
             'server_name': args.server_name,
             'display_name': args.display_name}
-
     if recurse_zones:
-        id_col = 'UUID'
+        to_print = ['UUID', 'Name', 'Status', 'Public IP', 'Private IP']
     else:
-        id_col = 'ID'
-
-    columns = [id_col, 'Name', 'Status', 'Networks']
-    formatters = {'Networks': _format_servers_list_networks}
-    utils.print_list(cs.servers.list(search_opts=search_opts), columns, formatters)
-
-
-def _format_servers_list_networks(server):
-    output = []
-    for (network, addresses) in server.networks.items():
-        if len(addresses) == 0:
-            continue
-        addresses_csv = ', '.join(addresses)
-        group = "%s=%s" % (network, addresses_csv)
-        output.append(group)
-
-    return '; '.join(output)
-
+        to_print = ['ID', 'Name', 'Status', 'Public IP', 'Private IP']
+    utils.print_list(cs.servers.list(search_opts=search_opts),
+            to_print)
 
 @utils.arg('--hard',
     dest='reboot_type',
@@ -343,14 +501,14 @@ def do_resize(cs, args):
     server.resize(flavor)
 
 @utils.arg('server', metavar='<server>', help='Name or ID of server.')
-def do_resize_confirm(cs, args):
-    """Confirm a previous resize."""
-    _find_server(cs, args.server).confirm_resize()
-
-@utils.arg('server', metavar='<server>', help='Name or ID of server.')
-def do_resize_revert(cs, args):
-    """Revert a previous resize (and return to the previous VM)."""
-    _find_server(cs, args.server).revert_resize()
+@utils.arg('name', metavar='<name>', help='Name of snapshot.')
+@utils.arg('backup_type', metavar='<daily|weekly>', help='type of backup')
+@utils.arg('rotation', type=int, metavar='<rotation>',
+     help="Number of backups to retain. Used for backup image_type.")
+def do_backup(cs, args):
+    """Resize a server."""
+    server = _find_server(cs, args.server)
+    server.backup(args.name, args.backup_type, args.rotation)
 
 @utils.arg('server', metavar='<server>', help='Name or ID of server.')
 def do_migrate(cs, args):
@@ -399,7 +557,15 @@ def do_actions(cs, args):
         cs.servers.actions(args.server),
         ["Created_At", "Action", "Error"])
 
+@utils.arg('server', metavar='<server>', help='Name or ID of server.')
+def do_resize_confirm(cs, args):
+    """Confirm a previous resize."""
+    _find_server(cs, args.server).confirm_resize()
 
+@utils.arg('server', metavar='<server>', help='Name or ID of server.')
+def do_resize_revert(cs, args):
+    """Revert a previous resize (and return to the previous VM)."""
+    _find_server(cs, args.server).revert_resize()
 
 @utils.arg('server', metavar='<server>', help='Name or ID of server.')
 def do_root_password(cs, args):
@@ -411,36 +577,24 @@ def do_root_password(cs, args):
     p2 = getpass.getpass('Again: ')
     if p1 != p2:
         raise exceptions.CommandError("Passwords do not match.")
-    server.change_password(p1)
-
-@utils.arg('server', metavar='<server>', help='Name or ID of server.')
-@utils.arg('name', metavar='<name>', help='Name of snapshot.')
-def do_image_create(cs, args):
-    """Create a new image by taking a snapshot of a running server."""
-    server = _find_server(cs, args.server)
-    cs.servers.create_image(server, args.name)
+    server.update(password=p1)
 
 @utils.arg('server', metavar='<server>', help='Name or ID of server.')
 def do_show(cs, args):
     """Show details about the given server."""
     s = _find_server(cs, args.server)
 
-    networks = s.networks
-
     info = s._info.copy()
-    for network_label, address_list in networks.items():
-        info['%s network' % network_label] = ', '.join(address_list)
+    addresses = info.pop('addresses')
+    for addrtype in addresses:
+        info['%s ip' % addrtype] = ', '.join(addresses[addrtype])
 
-    flavor = info.get('flavor', {})
-    flavor_id = flavor.get('id', '')
-    info['flavor'] = _find_flavor(cs, flavor_id).name
-
-    image = info.get('image', {})
-    image_id = image.get('id', '')
-    info['image'] = _find_image(cs, image_id).name
-
-    info.pop('links', None)
-    info.pop('addresses', None)
+    flavorId = info.get('flavorId', None)
+    if flavorId:
+        info['flavor'] = _find_flavor(cs, info.pop('flavorId')).name
+    imageId = info.get('imageId', None)
+    if imageId:
+        info['image'] = _find_image(cs, info.pop('imageId')).name
 
     utils.print_dict(info)
 
@@ -449,36 +603,6 @@ def do_delete(cs, args):
     """Immediately shut down and delete a server."""
     _find_server(cs, args.server).delete()
 
-def _find_server(cs, server):
-    """Get a server by name or ID."""
-    return _find_resource(cs.servers, server)
-
-def _find_image(cs, image):
-    """Get an image by name or ID."""
-    return _find_resource(cs.images, image)
-
-def _find_flavor(cs, flavor):
-    """Get a flavor by name, ID, or RAM size."""
-    try:
-        return _find_resource(cs.flavors, flavor)
-    except exceptions.NotFound:
-        return cs.flavors.find(ram=flavor)
-
-def _find_resource(manager, name_or_id):
-    """Helper for the _find_* methods."""
-    try:
-        if isinstance(name_or_id, int) or name_or_id.isdigit():
-            return manager.get(int(name_or_id))
-
-        try:
-            uuid.UUID(name_or_id)
-            return manager.get(name_or_id)
-        except ValueError:
-            return manager.find(name=name_or_id)
-    except exceptions.NotFound:
-        raise exceptions.CommandError("No %s with a name or ID of '%s' exists." %
-                     (manager.resource_class.__name__.lower(), name_or_id))
-
 # --zone_username is required since --username is already used.
 @utils.arg('zone', metavar='<zone_id>', help='ID of the zone', default=None)
 @utils.arg('--api_url', dest='api_url', default=None, help='New URL.')
@@ -553,3 +677,38 @@ def do_remove_fixed_ip(cs, args):
     """Remove an IP address from a server."""
     server = _find_server(cs, args.server)
     server.remove_fixed_ip(args.address)
+
+def _find_server(cs, server):
+    """Get a server by name or ID."""
+    return _find_resource(cs.servers, server)
+
+def _find_ipgroup(cs, group):
+    """Get an IP group by name or ID."""
+    return _find_resource(cs.ipgroups, group)
+
+def _find_image(cs, image):
+    """Get an image by name or ID."""
+    return _find_resource(cs.images, image)
+
+def _find_flavor(cs, flavor):
+    """Get a flavor by name, ID, or RAM size."""
+    try:
+        return _find_resource(cs.flavors, flavor)
+    except exceptions.NotFound:
+        return cs.flavors.find(ram=flavor)
+
+def _find_resource(manager, name_or_id):
+    """Helper for the _find_* methods."""
+    try:
+        if isinstance(name_or_id, int) or name_or_id.isdigit():
+            return manager.get(int(name_or_id))
+
+        try:
+            uuid.UUID(name_or_id)
+            return manager.get(name_or_id)
+        except ValueError:
+            return manager.find(name=name_or_id)
+    except exceptions.NotFound:
+        raise exceptions.CommandError("No %s with a name or ID of '%s' exists." %
+                     (manager.resource_class.__name__.lower(), name_or_id))
+
